"use strict";

function _interopDefault(ex) {
  return ex && "object" == typeof ex && "default" in ex ? ex.default : ex;
}

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var _objectSpread = _interopDefault(require("@babel/runtime/helpers/objectSpread")), spawn = _interopDefault(require("spawndamnit")), path = _interopDefault(require("path")), getWorkspaces = _interopDefault(require("get-workspaces")), pkgDir = _interopDefault(require("pkg-dir")), errors = require("@changesets/errors");

async function getProjectDirectory(cwd) {
  const projectDir = await pkgDir(cwd);
  if (!projectDir) throw new Error("Could not find project directory");
  return projectDir;
}

async function getMasterRef(cwd) {
  const gitCmd = await spawn("git", [ "rev-parse", "master" ], {
    cwd: cwd
  });
  if (0 !== gitCmd.code) throw new errors.GitError(gitCmd.code, gitCmd.stderr.toString());
  return gitCmd.stdout.toString().trim().split("\n")[0];
}

async function add(pathToFile, cwd) {
  const gitCmd = await spawn("git", [ "add", pathToFile ], {
    cwd: cwd
  });
  return 0 !== gitCmd.code && console.log(pathToFile, gitCmd.stderr.toString()), 0 === gitCmd.code;
}

async function commit(message, cwd) {
  return 0 === (await spawn("git", [ "commit", "-m", message, "--allow-empty" ], {
    cwd: cwd
  })).code;
}

async function tag(tagStr, cwd) {
  return 0 === (await spawn("git", [ "tag", tagStr, "-m", tagStr ], {
    cwd: cwd
  })).code;
}

async function getCommitThatAddsFile(gitPath, cwd) {
  return (await spawn("git", [ "log", "--reverse", "--max-count=1", "--pretty=format:%h", "-p", gitPath ], {
    cwd: cwd
  })).stdout.toString().split("\n")[0];
}

async function getChangedFilesSince(ref, cwd, fullPath = !1) {
  let cmd = await spawn("git", [ "merge-base", ref, "HEAD" ], {
    cwd: cwd
  });
  const divergedAt = cmd.stdout.toString().trim(), files = (cmd = await spawn("git", [ "diff", "--name-only", divergedAt ], {
    cwd: cwd
  })).stdout.toString().trim().split("\n");
  return fullPath ? files.map(file => path.resolve(cwd, file)) : files;
}

async function getChangedChangesetFilesSinceMaster(cwd, fullPath = !1) {
  try {
    const ref = await getMasterRef(cwd);
    let cmd = await spawn("git", [ "merge-base", ref, "HEAD" ], {
      cwd: cwd
    });
    cmd = await spawn("git", [ "diff", "--name-only", "--diff-filter=d", "master" ], {
      cwd: cwd
    });
    let tester = /.changeset\/[^\/]+\.md$/;
    const files = cmd.stdout.toString().trim().split("\n").filter(file => tester.test(file));
    return fullPath ? files.map(file => path.resolve(cwd, file)) : files;
  } catch (err) {
    if (err instanceof errors.GitError) return [];
    throw err;
  }
}

async function getChangedPackagesSinceCommit(commitHash, cwd) {
  const changedFiles = await getChangedFilesSince(commitHash, cwd, !0), projectDir = await getProjectDirectory(cwd);
  let workspaces = await getWorkspaces({
    cwd: cwd,
    tools: [ "yarn", "bolt", "root" ]
  });
  null === workspaces && (workspaces = []);
  const allPackages = workspaces.map(pkg => _objectSpread({}, pkg, {
    relativeDir: path.relative(projectDir, pkg.dir)
  })), fileNameToPackage = fileName => allPackages.find(pkg => fileName.startsWith(pkg.dir + path.sep));
  return changedFiles.filter(fileName => !!fileNameToPackage(fileName)).map(fileNameToPackage).filter((pkg, idx, packages) => packages.indexOf(pkg) === idx);
}

async function getChangedPackagesSinceMaster(cwd) {
  try {
    return getChangedPackagesSinceCommit(await getMasterRef(cwd), cwd);
  } catch (err) {
    if (err instanceof errors.GitError) return [];
    throw err;
  }
}

exports.add = add, exports.commit = commit, exports.getChangedChangesetFilesSinceMaster = getChangedChangesetFilesSinceMaster, 
exports.getChangedFilesSince = getChangedFilesSince, exports.getChangedPackagesSinceMaster = getChangedPackagesSinceMaster, 
exports.getCommitThatAddsFile = getCommitThatAddsFile, exports.tag = tag;
